from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.log_embedding_request_embeddings import LogEmbeddingRequestEmbeddings


T = TypeVar("T", bound="LogEmbeddingRequest")


@_attrs_define
class LogEmbeddingRequest:
    """
    Attributes:
        dataset_id (str):
        timestamp (int):
        embeddings (LogEmbeddingRequestEmbeddings):
    """

    dataset_id: str
    timestamp: int
    embeddings: "LogEmbeddingRequestEmbeddings"
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        dataset_id = self.dataset_id

        timestamp = self.timestamp

        embeddings = self.embeddings.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "dataset_id": dataset_id,
                "timestamp": timestamp,
                "embeddings": embeddings,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.log_embedding_request_embeddings import LogEmbeddingRequestEmbeddings

        d = src_dict.copy()
        dataset_id = d.pop("dataset_id")

        timestamp = d.pop("timestamp")

        embeddings = LogEmbeddingRequestEmbeddings.from_dict(d.pop("embeddings"))

        log_embedding_request = cls(
            dataset_id=dataset_id,
            timestamp=timestamp,
            embeddings=embeddings,
        )

        log_embedding_request.additional_properties = d
        return log_embedding_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.input_context_item_metadata import InputContextItemMetadata


T = TypeVar("T", bound="InputContextItem")


@_attrs_define
class InputContextItem:
    """
    Attributes:
        content (str):
        metadata (Union[Unset, InputContextItemMetadata]):
    """

    content: str
    metadata: Union[Unset, "InputContextItemMetadata"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        content = self.content

        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "content": content,
            }
        )
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.input_context_item_metadata import InputContextItemMetadata

        d = src_dict.copy()
        content = d.pop("content")

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, InputContextItemMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = InputContextItemMetadata.from_dict(_metadata)

        input_context_item = cls(
            content=content,
            metadata=metadata,
        )

        input_context_item.additional_properties = d
        return input_context_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



""" Contains all the data models used in inputs/outputs """

from .available_metrics import AvailableMetrics
from .block_action import BlockAction
from .debug_llm_validate_request import DebugLLMValidateRequest
from .debug_llm_validate_request_additional_data import DebugLLMValidateRequestAdditionalData
from .evaluation_result import EvaluationResult
from .evaluation_result_metrics_item import EvaluationResultMetricsItem
from .evaluation_result_scores_item import EvaluationResultScoresItem
from .http_validation_error import HTTPValidationError
from .input_context import InputContext
from .input_context_item import InputContextItem
from .input_context_item_metadata import InputContextItemMetadata
from .llm_validate_request import LLMValidateRequest
from .llm_validate_request_additional_data import LLMValidateRequestAdditionalData
from .log_embedding_request import LogEmbeddingRequest
from .log_embedding_request_embeddings import LogEmbeddingRequestEmbeddings
from .log_multiple import LogMultiple
from .log_request import LogRequest
from .logger_status_response import LoggerStatusResponse
from .metric_filter_options import MetricFilterOptions
from .pass_action import PassAction
from .policy_response_policy import PolicyResponsePolicy
from .process_logger_status_response import ProcessLoggerStatusResponse
from .process_logger_status_response_statuses import ProcessLoggerStatusResponseStatuses
from .run_options import RunOptions
from .run_perf import RunPerf
from .run_perf_context_time_sec import RunPerfContextTimeSec
from .run_perf_metrics_time_sec import RunPerfMetricsTimeSec
from .validation_error import ValidationError
from .validation_failure import ValidationFailure
from .validation_failure_failure_level import ValidationFailureFailureLevel
from .validation_result import ValidationResult

__all__ = (
    "AvailableMetrics",
    "BlockAction",
    "DebugLLMValidateRequest",
    "DebugLLMValidateRequestAdditionalData",
    "EvaluationResult",
    "EvaluationResultMetricsItem",
    "EvaluationResultScoresItem",
    "HTTPValidationError",
    "InputContext",
    "InputContextItem",
    "InputContextItemMetadata",
    "LLMValidateRequest",
    "LLMValidateRequestAdditionalData",
    "LogEmbeddingRequest",
    "LogEmbeddingRequestEmbeddings",
    "LoggerStatusResponse",
    "LogMultiple",
    "LogRequest",
    "MetricFilterOptions",
    "PassAction",
    "PolicyResponsePolicy",
    "ProcessLoggerStatusResponse",
    "ProcessLoggerStatusResponseStatuses",
    "RunOptions",
    "RunPerf",
    "RunPerfContextTimeSec",
    "RunPerfMetricsTimeSec",
    "ValidationError",
    "ValidationFailure",
    "ValidationFailureFailureLevel",
    "ValidationResult",
)



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...models.log_request import LogRequest
from ...types import Response


def _get_kwargs(
    *,
    body: LogRequest,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}

    _kwargs: Dict[str, Any] = {
        "method": "post",
        "url": "/log",
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, HTTPValidationError]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = response.json()
        return response_200
    if response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, HTTPValidationError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: LogRequest,
) -> Response[Union[Any, HTTPValidationError]]:
    r"""Profile tabular data

     Profile tabular data. The Swagger UI isn't able to call this currently.

    ## Sample curl request:

    ```bash
    curl -X 'POST'         -H \"X-API-Key: <password>\"         -H \"Content-Type: application/json\"
    'http://localhost:8000/log'         --data-raw '{
        \"datasetId\": \"model-62\",
        \"multiple\": {
            \"columns\": [ \"age\", \"workclass\", \"fnlwgt\", \"education\" ],
            \"data\": [
                [ 25, \"Private\", 226802, \"11th\" ]
            ]
        }
    }'
    ```
    ## Sample Python client request:
    ```python
    from whylogs_container_client import AuthenticatedClient
    import whylogs_container_client.api.profile.log as Log
    from whylogs_container_client.models import LogRequest, LogMultiple
    from datetime import datetime

    client = AuthenticatedClient(base_url=\"http://localhost:8000\", token=\"password\", prefix=\"\",
    auth_header_name=\"X-API-Key\")

    data = LogRequest(
        dataset_id=\"model-1\",
        timestamp=int(datetime.now().timestamp() * 1000),
        multiple=LogMultiple(
            columns=[\"col1\", \"col2\"],
            data=[[1, 2], [3, 4]],
        )
    )

    response = Log.sync_detailed(client=client, json_body=data)
    if response.status_code != 200:
        raise Exception(f\"Failed to log data. Status code: {response.status_code}\")
    # API is async, it won't fail and has no return body
    ```

    ## Sample Python request (using `requests`):
    ```python
    import requests

    # Define your API key
    api_key = \"<password>\"

    # API endpoint
    url = 'http://localhost:8000/log'

    # Sample data
    data = {
        \"datasetId\": \"model-62\",
        \"multiple\": {
            \"columns\": [\"age\", \"workclass\", \"fnlwgt\", \"education\"],
            \"data\": [
                [25, \"Private\", 226802, \"11th\"]
            ]
        }
    }

    # Make the POST request
    headers = {\"X-API-Key\": api_key}
    response = requests.post(url, json=data, headers=headers)
    ```

    Args:
        body (LogRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, HTTPValidationError]]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    body: LogRequest,
) -> Optional[Union[Any, HTTPValidationError]]:
    r"""Profile tabular data

     Profile tabular data. The Swagger UI isn't able to call this currently.

    ## Sample curl request:

    ```bash
    curl -X 'POST'         -H \"X-API-Key: <password>\"         -H \"Content-Type: application/json\"
    'http://localhost:8000/log'         --data-raw '{
        \"datasetId\": \"model-62\",
        \"multiple\": {
            \"columns\": [ \"age\", \"workclass\", \"fnlwgt\", \"education\" ],
            \"data\": [
                [ 25, \"Private\", 226802, \"11th\" ]
            ]
        }
    }'
    ```
    ## Sample Python client request:
    ```python
    from whylogs_container_client import AuthenticatedClient
    import whylogs_container_client.api.profile.log as Log
    from whylogs_container_client.models import LogRequest, LogMultiple
    from datetime import datetime

    client = AuthenticatedClient(base_url=\"http://localhost:8000\", token=\"password\", prefix=\"\",
    auth_header_name=\"X-API-Key\")

    data = LogRequest(
        dataset_id=\"model-1\",
        timestamp=int(datetime.now().timestamp() * 1000),
        multiple=LogMultiple(
            columns=[\"col1\", \"col2\"],
            data=[[1, 2], [3, 4]],
        )
    )

    response = Log.sync_detailed(client=client, json_body=data)
    if response.status_code != 200:
        raise Exception(f\"Failed to log data. Status code: {response.status_code}\")
    # API is async, it won't fail and has no return body
    ```

    ## Sample Python request (using `requests`):
    ```python
    import requests

    # Define your API key
    api_key = \"<password>\"

    # API endpoint
    url = 'http://localhost:8000/log'

    # Sample data
    data = {
        \"datasetId\": \"model-62\",
        \"multiple\": {
            \"columns\": [\"age\", \"workclass\", \"fnlwgt\", \"education\"],
            \"data\": [
                [25, \"Private\", 226802, \"11th\"]
            ]
        }
    }

    # Make the POST request
    headers = {\"X-API-Key\": api_key}
    response = requests.post(url, json=data, headers=headers)
    ```

    Args:
        body (LogRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, HTTPValidationError]
    """

    return sync_detailed(
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: LogRequest,
) -> Response[Union[Any, HTTPValidationError]]:
    r"""Profile tabular data

     Profile tabular data. The Swagger UI isn't able to call this currently.

    ## Sample curl request:

    ```bash
    curl -X 'POST'         -H \"X-API-Key: <password>\"         -H \"Content-Type: application/json\"
    'http://localhost:8000/log'         --data-raw '{
        \"datasetId\": \"model-62\",
        \"multiple\": {
            \"columns\": [ \"age\", \"workclass\", \"fnlwgt\", \"education\" ],
            \"data\": [
                [ 25, \"Private\", 226802, \"11th\" ]
            ]
        }
    }'
    ```
    ## Sample Python client request:
    ```python
    from whylogs_container_client import AuthenticatedClient
    import whylogs_container_client.api.profile.log as Log
    from whylogs_container_client.models import LogRequest, LogMultiple
    from datetime import datetime

    client = AuthenticatedClient(base_url=\"http://localhost:8000\", token=\"password\", prefix=\"\",
    auth_header_name=\"X-API-Key\")

    data = LogRequest(
        dataset_id=\"model-1\",
        timestamp=int(datetime.now().timestamp() * 1000),
        multiple=LogMultiple(
            columns=[\"col1\", \"col2\"],
            data=[[1, 2], [3, 4]],
        )
    )

    response = Log.sync_detailed(client=client, json_body=data)
    if response.status_code != 200:
        raise Exception(f\"Failed to log data. Status code: {response.status_code}\")
    # API is async, it won't fail and has no return body
    ```

    ## Sample Python request (using `requests`):
    ```python
    import requests

    # Define your API key
    api_key = \"<password>\"

    # API endpoint
    url = 'http://localhost:8000/log'

    # Sample data
    data = {
        \"datasetId\": \"model-62\",
        \"multiple\": {
            \"columns\": [\"age\", \"workclass\", \"fnlwgt\", \"education\"],
            \"data\": [
                [25, \"Private\", 226802, \"11th\"]
            ]
        }
    }

    # Make the POST request
    headers = {\"X-API-Key\": api_key}
    response = requests.post(url, json=data, headers=headers)
    ```

    Args:
        body (LogRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, HTTPValidationError]]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    body: LogRequest,
) -> Optional[Union[Any, HTTPValidationError]]:
    r"""Profile tabular data

     Profile tabular data. The Swagger UI isn't able to call this currently.

    ## Sample curl request:

    ```bash
    curl -X 'POST'         -H \"X-API-Key: <password>\"         -H \"Content-Type: application/json\"
    'http://localhost:8000/log'         --data-raw '{
        \"datasetId\": \"model-62\",
        \"multiple\": {
            \"columns\": [ \"age\", \"workclass\", \"fnlwgt\", \"education\" ],
            \"data\": [
                [ 25, \"Private\", 226802, \"11th\" ]
            ]
        }
    }'
    ```
    ## Sample Python client request:
    ```python
    from whylogs_container_client import AuthenticatedClient
    import whylogs_container_client.api.profile.log as Log
    from whylogs_container_client.models import LogRequest, LogMultiple
    from datetime import datetime

    client = AuthenticatedClient(base_url=\"http://localhost:8000\", token=\"password\", prefix=\"\",
    auth_header_name=\"X-API-Key\")

    data = LogRequest(
        dataset_id=\"model-1\",
        timestamp=int(datetime.now().timestamp() * 1000),
        multiple=LogMultiple(
            columns=[\"col1\", \"col2\"],
            data=[[1, 2], [3, 4]],
        )
    )

    response = Log.sync_detailed(client=client, json_body=data)
    if response.status_code != 200:
        raise Exception(f\"Failed to log data. Status code: {response.status_code}\")
    # API is async, it won't fail and has no return body
    ```

    ## Sample Python request (using `requests`):
    ```python
    import requests

    # Define your API key
    api_key = \"<password>\"

    # API endpoint
    url = 'http://localhost:8000/log'

    # Sample data
    data = {
        \"datasetId\": \"model-62\",
        \"multiple\": {
            \"columns\": [\"age\", \"workclass\", \"fnlwgt\", \"education\"],
            \"data\": [
                [25, \"Private\", 226802, \"11th\"]
            ]
        }
    }

    # Make the POST request
    headers = {\"X-API-Key\": api_key}
    response = requests.post(url, json=data, headers=headers)
    ```

    Args:
        body (LogRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, HTTPValidationError]
    """

    return (
        await asyncio_detailed(
            client=client,
            body=body,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET

T = TypeVar("T", bound="LogEmbeddingRequestEmbeddings")


@_attrs_define
class LogEmbeddingRequestEmbeddings:
    """ """

    additional_properties: Dict[str, Union[List[List[float]], List[List[int]], List[List[str]]]] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        field_dict: Dict[str, Any] = {}
        for prop_name, prop in self.additional_properties.items():
            if isinstance(prop, list):
                field_dict[prop_name] = []
                for additional_property_type_0_item_data in prop:
                    additional_property_type_0_item = additional_property_type_0_item_data

                    field_dict[prop_name].append(additional_property_type_0_item)

            elif isinstance(prop, list):
                field_dict[prop_name] = []
                for additional_property_type_1_item_data in prop:
                    additional_property_type_1_item = additional_property_type_1_item_data

                    field_dict[prop_name].append(additional_property_type_1_item)

            else:
                field_dict[prop_name] = []
                for additional_property_type_2_item_data in prop:
                    additional_property_type_2_item = additional_property_type_2_item_data

                    field_dict[prop_name].append(additional_property_type_2_item)

        field_dict.update({})

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        log_embedding_request_embeddings = cls()

        additional_properties = {}
        for prop_name, prop_dict in d.items():

            def _parse_additional_property(data: object) -> Union[List[List[float]], List[List[int]], List[List[str]]]:
                try:
                    if not isinstance(data, list):
                        raise TypeError()
                    additional_property_type_0 = UNSET
                    _additional_property_type_0 = data
                    for additional_property_type_0_item_data in _additional_property_type_0:
                        additional_property_type_0_item = cast(List[float], additional_property_type_0_item_data)

                        additional_property_type_0.append(additional_property_type_0_item)

                    return additional_property_type_0
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, list):
                        raise TypeError()
                    additional_property_type_1 = UNSET
                    _additional_property_type_1 = data
                    for additional_property_type_1_item_data in _additional_property_type_1:
                        additional_property_type_1_item = cast(List[int], additional_property_type_1_item_data)

                        additional_property_type_1.append(additional_property_type_1_item)

                    return additional_property_type_1
                except:  # noqa: E722
                    pass
                if not isinstance(data, list):
                    raise TypeError()
                additional_property_type_2 = UNSET
                _additional_property_type_2 = data
                for additional_property_type_2_item_data in _additional_property_type_2:
                    additional_property_type_2_item = cast(List[str], additional_property_type_2_item_data)

                    additional_property_type_2.append(additional_property_type_2_item)

                return additional_property_type_2

            additional_property = _parse_additional_property(prop_dict)

            additional_properties[prop_name] = additional_property

        log_embedding_request_embeddings.additional_properties = additional_properties
        return log_embedding_request_embeddings

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Union[List[List[float]], List[List[int]], List[List[str]]]:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Union[List[List[float]], List[List[int]], List[List[str]]]) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.process_logger_status_response_statuses import ProcessLoggerStatusResponseStatuses


T = TypeVar("T", bound="ProcessLoggerStatusResponse")


@_attrs_define
class ProcessLoggerStatusResponse:
    """
    Attributes:
        version (str):
        statuses (ProcessLoggerStatusResponseStatuses):
    """

    version: str
    statuses: "ProcessLoggerStatusResponseStatuses"
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        version = self.version

        statuses = self.statuses.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "version": version,
                "statuses": statuses,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.process_logger_status_response_statuses import ProcessLoggerStatusResponseStatuses

        d = src_dict.copy()
        version = d.pop("version")

        statuses = ProcessLoggerStatusResponseStatuses.from_dict(d.pop("statuses"))

        process_logger_status_response = cls(
            version=version,
            statuses=statuses,
        )

        process_logger_status_response.additional_properties = d
        return process_logger_status_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="MetricFilterOptions")


@_attrs_define
class MetricFilterOptions:
    """
    Attributes:
        by_required_inputs (Union[List[List[str]], None, Unset]):
    """

    by_required_inputs: Union[List[List[str]], None, Unset] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        by_required_inputs: Union[List[List[str]], None, Unset]
        if isinstance(self.by_required_inputs, Unset):
            by_required_inputs = UNSET
        elif isinstance(self.by_required_inputs, list):
            by_required_inputs = UNSET
            if not isinstance(self.by_required_inputs, Unset):
                by_required_inputs = []
                for by_required_inputs_type_0_item_data in self.by_required_inputs:
                    by_required_inputs_type_0_item = by_required_inputs_type_0_item_data

                    by_required_inputs.append(by_required_inputs_type_0_item)

        else:
            by_required_inputs = self.by_required_inputs

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if by_required_inputs is not UNSET:
            field_dict["by_required_inputs"] = by_required_inputs

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_by_required_inputs(data: object) -> Union[List[List[str]], None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                by_required_inputs_type_0 = UNSET
                _by_required_inputs_type_0 = data
                for by_required_inputs_type_0_item_data in _by_required_inputs_type_0 or []:
                    by_required_inputs_type_0_item = cast(List[str], by_required_inputs_type_0_item_data)

                    by_required_inputs_type_0.append(by_required_inputs_type_0_item)

                return by_required_inputs_type_0
            except:  # noqa: E722
                pass
            return cast(Union[List[List[str]], None, Unset], data)

        by_required_inputs = _parse_by_required_inputs(d.pop("by_required_inputs", UNSET))

        metric_filter_options = cls(
            by_required_inputs=by_required_inputs,
        )

        metric_filter_options.additional_properties = d
        return metric_filter_options

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.metric_filter_options import MetricFilterOptions


T = TypeVar("T", bound="RunOptions")


@_attrs_define
class RunOptions:
    """
    Attributes:
        metric_filter (Union['MetricFilterOptions', None, Unset]):
    """

    metric_filter: Union["MetricFilterOptions", None, Unset] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.metric_filter_options import MetricFilterOptions

        metric_filter: Union[Dict[str, Any], None, Unset]
        if isinstance(self.metric_filter, Unset):
            metric_filter = UNSET
        elif isinstance(self.metric_filter, MetricFilterOptions):
            metric_filter = UNSET
            if not isinstance(self.metric_filter, Unset):
                metric_filter = self.metric_filter.to_dict()
        else:
            metric_filter = self.metric_filter

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if metric_filter is not UNSET:
            field_dict["metric_filter"] = metric_filter

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.metric_filter_options import MetricFilterOptions

        d = src_dict.copy()

        def _parse_metric_filter(data: object) -> Union["MetricFilterOptions", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                _metric_filter_type_0 = data
                metric_filter_type_0: Union[Unset, MetricFilterOptions]
                if isinstance(_metric_filter_type_0, Unset):
                    metric_filter_type_0 = UNSET
                else:
                    metric_filter_type_0 = MetricFilterOptions.from_dict(_metric_filter_type_0)

                return metric_filter_type_0
            except:  # noqa: E722
                pass
            return cast(Union["MetricFilterOptions", None, Unset], data)

        metric_filter = _parse_metric_filter(d.pop("metric_filter", UNSET))

        run_options = cls(
            metric_filter=metric_filter,
        )

        run_options.additional_properties = d
        return run_options

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.validation_failure import ValidationFailure


T = TypeVar("T", bound="ValidationResult")


@_attrs_define
class ValidationResult:
    """
    Attributes:
        report (Union[Unset, List['ValidationFailure']]):
    """

    report: Union[Unset, List["ValidationFailure"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        report: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.report, Unset):
            report = []
            for report_item_data in self.report:
                report_item = report_item_data.to_dict()
                report.append(report_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if report is not UNSET:
            field_dict["report"] = report

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.validation_failure import ValidationFailure

        d = src_dict.copy()
        report = []
        _report = d.pop("report", UNSET)
        for report_item_data in _report or []:
            report_item = ValidationFailure.from_dict(report_item_data)

            report.append(report_item)

        validation_result = cls(
            report=report,
        )

        validation_result.additional_properties = d
        return validation_result

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Literal, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="PassAction")


@_attrs_define
class PassAction:
    """
    Attributes:
        is_action_pass (bool):
        action_type (Union[Literal['pass'], Unset]):  Default: 'pass'.
    """

    is_action_pass: bool
    action_type: Union[Literal["pass"], Unset] = "pass"
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        is_action_pass = self.is_action_pass

        action_type = self.action_type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "is_action_pass": is_action_pass,
            }
        )
        if action_type is not UNSET:
            field_dict["action_type"] = action_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        is_action_pass = d.pop("is_action_pass")

        action_type = d.pop("action_type", UNSET)

        pass_action = cls(
            is_action_pass=is_action_pass,
            action_type=action_type,
        )

        pass_action.additional_properties = d
        return pass_action

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.log_multiple import LogMultiple


T = TypeVar("T", bound="LogRequest")


@_attrs_define
class LogRequest:
    """
    Attributes:
        dataset_id (str):
        multiple (LogMultiple):
        timestamp (Union[None, Unset, int]):
    """

    dataset_id: str
    multiple: "LogMultiple"
    timestamp: Union[None, Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        dataset_id = self.dataset_id

        multiple = self.multiple.to_dict()

        timestamp: Union[None, Unset, int]
        if isinstance(self.timestamp, Unset):
            timestamp = UNSET
        else:
            timestamp = self.timestamp

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "datasetId": dataset_id,
                "multiple": multiple,
            }
        )
        if timestamp is not UNSET:
            field_dict["timestamp"] = timestamp

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.log_multiple import LogMultiple

        d = src_dict.copy()
        dataset_id = d.pop("datasetId")

        multiple = LogMultiple.from_dict(d.pop("multiple"))

        def _parse_timestamp(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        timestamp = _parse_timestamp(d.pop("timestamp", UNSET))

        log_request = cls(
            dataset_id=dataset_id,
            multiple=multiple,
            timestamp=timestamp,
        )

        log_request.additional_properties = d
        return log_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="LogMultiple")


@_attrs_define
class LogMultiple:
    """
    Attributes:
        columns (List[str]):
        data (List[List[Union[List[float], List[int], List[str], None, bool, float, int, str]]]):
    """

    columns: List[str]
    data: List[List[Union[List[float], List[int], List[str], None, bool, float, int, str]]]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        columns = self.columns

        data = []
        for data_item_data in self.data:
            data_item = []
            for data_item_item_data in data_item_data:
                data_item_item: Union[List[float], List[int], List[str], None, bool, float, int, str]
                if isinstance(data_item_item_data, list):
                    data_item_item = data_item_item_data

                elif isinstance(data_item_item_data, list):
                    data_item_item = data_item_item_data

                elif isinstance(data_item_item_data, list):
                    data_item_item = data_item_item_data

                else:
                    data_item_item = data_item_item_data
                data_item.append(data_item_item)

            data.append(data_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "columns": columns,
                "data": data,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        columns = cast(List[str], d.pop("columns"))

        data = []
        _data = d.pop("data")
        for data_item_data in _data:
            data_item = []
            _data_item = data_item_data
            for data_item_item_data in _data_item:

                def _parse_data_item_item(
                    data: object,
                ) -> Union[List[float], List[int], List[str], None, bool, float, int, str]:
                    if data is None:
                        return data
                    try:
                        if not isinstance(data, list):
                            raise TypeError()
                        data_item_item_type_4 = cast(List[float], data)

                        return data_item_item_type_4
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, list):
                            raise TypeError()
                        data_item_item_type_5 = cast(List[int], data)

                        return data_item_item_type_5
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, list):
                            raise TypeError()
                        data_item_item_type_6 = cast(List[str], data)

                        return data_item_item_type_6
                    except:  # noqa: E722
                        pass
                    return cast(Union[List[float], List[int], List[str], None, bool, float, int, str], data)

                data_item_item = _parse_data_item_item(data_item_item_data)

                data_item.append(data_item_item)

            data.append(data_item)

        log_multiple = cls(
            columns=columns,
            data=data,
        )

        log_multiple.additional_properties = d
        return log_multiple

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.block_action import BlockAction
    from ..models.evaluation_result_metrics_item import EvaluationResultMetricsItem
    from ..models.evaluation_result_scores_item import EvaluationResultScoresItem
    from ..models.pass_action import PassAction
    from ..models.run_perf import RunPerf
    from ..models.validation_result import ValidationResult


T = TypeVar("T", bound="EvaluationResult")


@_attrs_define
class EvaluationResult:
    """
    Attributes:
        metrics (List['EvaluationResultMetricsItem']):
        validation_results (ValidationResult):
        perf_info (Union['RunPerf', None]):
        score_perf_info (Union['RunPerf', None]):
        action (Union['BlockAction', 'PassAction']):
        scores (Union[Unset, List['EvaluationResultScoresItem']]):
    """

    metrics: List["EvaluationResultMetricsItem"]
    validation_results: "ValidationResult"
    perf_info: Union["RunPerf", None]
    score_perf_info: Union["RunPerf", None]
    action: Union["BlockAction", "PassAction"]
    scores: Union[Unset, List["EvaluationResultScoresItem"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.pass_action import PassAction
        from ..models.run_perf import RunPerf

        metrics = []
        for metrics_item_data in self.metrics:
            metrics_item = metrics_item_data.to_dict()
            metrics.append(metrics_item)

        validation_results = self.validation_results.to_dict()

        perf_info: Union[Dict[str, Any], None]
        if isinstance(self.perf_info, RunPerf):
            perf_info = self.perf_info.to_dict()
        else:
            perf_info = self.perf_info

        score_perf_info: Union[Dict[str, Any], None]
        if isinstance(self.score_perf_info, RunPerf):
            score_perf_info = self.score_perf_info.to_dict()
        else:
            score_perf_info = self.score_perf_info

        action: Dict[str, Any]
        if isinstance(self.action, PassAction):
            action = self.action.to_dict()
        else:
            action = self.action.to_dict()

        scores: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.scores, Unset):
            scores = []
            for scores_item_data in self.scores:
                scores_item = scores_item_data.to_dict()
                scores.append(scores_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "metrics": metrics,
                "validation_results": validation_results,
                "perf_info": perf_info,
                "score_perf_info": score_perf_info,
                "action": action,
            }
        )
        if scores is not UNSET:
            field_dict["scores"] = scores

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.block_action import BlockAction
        from ..models.evaluation_result_metrics_item import EvaluationResultMetricsItem
        from ..models.evaluation_result_scores_item import EvaluationResultScoresItem
        from ..models.pass_action import PassAction
        from ..models.run_perf import RunPerf
        from ..models.validation_result import ValidationResult

        d = src_dict.copy()
        metrics = []
        _metrics = d.pop("metrics")
        for metrics_item_data in _metrics:
            metrics_item = EvaluationResultMetricsItem.from_dict(metrics_item_data)

            metrics.append(metrics_item)

        validation_results = ValidationResult.from_dict(d.pop("validation_results"))

        def _parse_perf_info(data: object) -> Union["RunPerf", None]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                perf_info_type_0 = RunPerf.from_dict(data)

                return perf_info_type_0
            except:  # noqa: E722
                pass
            return cast(Union["RunPerf", None], data)

        perf_info = _parse_perf_info(d.pop("perf_info"))

        def _parse_score_perf_info(data: object) -> Union["RunPerf", None]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                score_perf_info_type_0 = RunPerf.from_dict(data)

                return score_perf_info_type_0
            except:  # noqa: E722
                pass
            return cast(Union["RunPerf", None], data)

        score_perf_info = _parse_score_perf_info(d.pop("score_perf_info"))

        def _parse_action(data: object) -> Union["BlockAction", "PassAction"]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                action_type_0 = PassAction.from_dict(data)

                return action_type_0
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            action_type_1 = BlockAction.from_dict(data)

            return action_type_1

        action = _parse_action(d.pop("action"))

        scores = []
        _scores = d.pop("scores", UNSET)
        for scores_item_data in _scores or []:
            scores_item = EvaluationResultScoresItem.from_dict(scores_item_data)

            scores.append(scores_item)

        evaluation_result = cls(
            metrics=metrics,
            validation_results=validation_results,
            perf_info=perf_info,
            score_perf_info=score_perf_info,
            action=action,
            scores=scores,
        )

        evaluation_result.additional_properties = d
        return evaluation_result

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.validation_failure_failure_level import ValidationFailureFailureLevel
from ..types import UNSET, Unset

T = TypeVar("T", bound="ValidationFailure")


@_attrs_define
class ValidationFailure:
    """
    Attributes:
        id (str):
        metric (str):
        details (str):
        value (Union[None, float, int, str]):
        upper_threshold (Union[None, Unset, float]):
        lower_threshold (Union[None, Unset, float]):
        allowed_values (Union[List[Union[float, int, str]], None, Unset]):
        disallowed_values (Union[List[Union[float, int, str]], None, Unset]):
        must_be_none (Union[None, Unset, bool]):
        must_be_non_none (Union[None, Unset, bool]):
        failure_level (Union[Unset, ValidationFailureFailureLevel]):  Default: ValidationFailureFailureLevel.BLOCK.
    """

    id: str
    metric: str
    details: str
    value: Union[None, float, int, str]
    upper_threshold: Union[None, Unset, float] = UNSET
    lower_threshold: Union[None, Unset, float] = UNSET
    allowed_values: Union[List[Union[float, int, str]], None, Unset] = UNSET
    disallowed_values: Union[List[Union[float, int, str]], None, Unset] = UNSET
    must_be_none: Union[None, Unset, bool] = UNSET
    must_be_non_none: Union[None, Unset, bool] = UNSET
    failure_level: Union[Unset, ValidationFailureFailureLevel] = ValidationFailureFailureLevel.BLOCK
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        metric = self.metric

        details = self.details

        value: Union[None, float, int, str]
        value = self.value

        upper_threshold: Union[None, Unset, float]
        if isinstance(self.upper_threshold, Unset):
            upper_threshold = UNSET
        else:
            upper_threshold = self.upper_threshold

        lower_threshold: Union[None, Unset, float]
        if isinstance(self.lower_threshold, Unset):
            lower_threshold = UNSET
        else:
            lower_threshold = self.lower_threshold

        allowed_values: Union[List[Union[float, int, str]], None, Unset]
        if isinstance(self.allowed_values, Unset):
            allowed_values = UNSET
        elif isinstance(self.allowed_values, list):
            allowed_values = UNSET
            if not isinstance(self.allowed_values, Unset):
                allowed_values = []
                for allowed_values_type_0_item_data in self.allowed_values:
                    allowed_values_type_0_item: Union[float, int, str]
                    allowed_values_type_0_item = allowed_values_type_0_item_data
                    allowed_values.append(allowed_values_type_0_item)

        else:
            allowed_values = self.allowed_values

        disallowed_values: Union[List[Union[float, int, str]], None, Unset]
        if isinstance(self.disallowed_values, Unset):
            disallowed_values = UNSET
        elif isinstance(self.disallowed_values, list):
            disallowed_values = UNSET
            if not isinstance(self.disallowed_values, Unset):
                disallowed_values = []
                for disallowed_values_type_0_item_data in self.disallowed_values:
                    disallowed_values_type_0_item: Union[float, int, str]
                    disallowed_values_type_0_item = disallowed_values_type_0_item_data
                    disallowed_values.append(disallowed_values_type_0_item)

        else:
            disallowed_values = self.disallowed_values

        must_be_none: Union[None, Unset, bool]
        if isinstance(self.must_be_none, Unset):
            must_be_none = UNSET
        else:
            must_be_none = self.must_be_none

        must_be_non_none: Union[None, Unset, bool]
        if isinstance(self.must_be_non_none, Unset):
            must_be_non_none = UNSET
        else:
            must_be_non_none = self.must_be_non_none

        failure_level: Union[Unset, str] = UNSET
        if not isinstance(self.failure_level, Unset):
            failure_level = self.failure_level.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "metric": metric,
                "details": details,
                "value": value,
            }
        )
        if upper_threshold is not UNSET:
            field_dict["upper_threshold"] = upper_threshold
        if lower_threshold is not UNSET:
            field_dict["lower_threshold"] = lower_threshold
        if allowed_values is not UNSET:
            field_dict["allowed_values"] = allowed_values
        if disallowed_values is not UNSET:
            field_dict["disallowed_values"] = disallowed_values
        if must_be_none is not UNSET:
            field_dict["must_be_none"] = must_be_none
        if must_be_non_none is not UNSET:
            field_dict["must_be_non_none"] = must_be_non_none
        if failure_level is not UNSET:
            field_dict["failure_level"] = failure_level

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        metric = d.pop("metric")

        details = d.pop("details")

        def _parse_value(data: object) -> Union[None, float, int, str]:
            if data is None:
                return data
            return cast(Union[None, float, int, str], data)

        value = _parse_value(d.pop("value"))

        def _parse_upper_threshold(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        upper_threshold = _parse_upper_threshold(d.pop("upper_threshold", UNSET))

        def _parse_lower_threshold(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        lower_threshold = _parse_lower_threshold(d.pop("lower_threshold", UNSET))

        def _parse_allowed_values(data: object) -> Union[List[Union[float, int, str]], None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                allowed_values_type_0 = UNSET
                _allowed_values_type_0 = data
                for allowed_values_type_0_item_data in _allowed_values_type_0 or []:

                    def _parse_allowed_values_type_0_item(data: object) -> Union[float, int, str]:
                        return cast(Union[float, int, str], data)

                    allowed_values_type_0_item = _parse_allowed_values_type_0_item(allowed_values_type_0_item_data)

                    allowed_values_type_0.append(allowed_values_type_0_item)

                return allowed_values_type_0
            except:  # noqa: E722
                pass
            return cast(Union[List[Union[float, int, str]], None, Unset], data)

        allowed_values = _parse_allowed_values(d.pop("allowed_values", UNSET))

        def _parse_disallowed_values(data: object) -> Union[List[Union[float, int, str]], None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                disallowed_values_type_0 = UNSET
                _disallowed_values_type_0 = data
                for disallowed_values_type_0_item_data in _disallowed_values_type_0 or []:

                    def _parse_disallowed_values_type_0_item(data: object) -> Union[float, int, str]:
                        return cast(Union[float, int, str], data)

                    disallowed_values_type_0_item = _parse_disallowed_values_type_0_item(
                        disallowed_values_type_0_item_data
                    )

                    disallowed_values_type_0.append(disallowed_values_type_0_item)

                return disallowed_values_type_0
            except:  # noqa: E722
                pass
            return cast(Union[List[Union[float, int, str]], None, Unset], data)

        disallowed_values = _parse_disallowed_values(d.pop("disallowed_values", UNSET))

        def _parse_must_be_none(data: object) -> Union[None, Unset, bool]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, bool], data)

        must_be_none = _parse_must_be_none(d.pop("must_be_none", UNSET))

        def _parse_must_be_non_none(data: object) -> Union[None, Unset, bool]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, bool], data)

        must_be_non_none = _parse_must_be_non_none(d.pop("must_be_non_none", UNSET))

        _failure_level = d.pop("failure_level", UNSET)
        failure_level: Union[Unset, ValidationFailureFailureLevel]
        if isinstance(_failure_level, Unset):
            failure_level = UNSET
        else:
            failure_level = ValidationFailureFailureLevel(_failure_level)

        validation_failure = cls(
            id=id,
            metric=metric,
            details=details,
            value=value,
            upper_threshold=upper_threshold,
            lower_threshold=lower_threshold,
            allowed_values=allowed_values,
            disallowed_values=disallowed_values,
            must_be_none=must_be_none,
            must_be_non_none=must_be_non_none,
            failure_level=failure_level,
        )

        validation_failure.additional_properties = d
        return validation_failure

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.input_context_item import InputContextItem


T = TypeVar("T", bound="InputContext")


@_attrs_define
class InputContext:
    """
    Attributes:
        entries (List['InputContextItem']):
    """

    entries: List["InputContextItem"]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        entries = []
        for entries_item_data in self.entries:
            entries_item = entries_item_data.to_dict()
            entries.append(entries_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "entries": entries,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.input_context_item import InputContextItem

        d = src_dict.copy()
        entries = []
        _entries = d.pop("entries")
        for entries_item_data in _entries:
            entries_item = InputContextItem.from_dict(entries_item_data)

            entries.append(entries_item)

        input_context = cls(
            entries=entries,
        )

        input_context.additional_properties = d
        return input_context

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.run_perf_context_time_sec import RunPerfContextTimeSec
    from ..models.run_perf_metrics_time_sec import RunPerfMetricsTimeSec


T = TypeVar("T", bound="RunPerf")


@_attrs_define
class RunPerf:
    """
    Attributes:
        init_total_sec (float):
        metrics_time_sec (RunPerfMetricsTimeSec):
        metrics_total_sec (float):
        context_time_sec (RunPerfContextTimeSec):
        context_total_sec (float):
        validation_total_sec (float):
        workflow_total_sec (float):
    """

    init_total_sec: float
    metrics_time_sec: "RunPerfMetricsTimeSec"
    metrics_total_sec: float
    context_time_sec: "RunPerfContextTimeSec"
    context_total_sec: float
    validation_total_sec: float
    workflow_total_sec: float
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        init_total_sec = self.init_total_sec

        metrics_time_sec = self.metrics_time_sec.to_dict()

        metrics_total_sec = self.metrics_total_sec

        context_time_sec = self.context_time_sec.to_dict()

        context_total_sec = self.context_total_sec

        validation_total_sec = self.validation_total_sec

        workflow_total_sec = self.workflow_total_sec

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "init_total_sec": init_total_sec,
                "metrics_time_sec": metrics_time_sec,
                "metrics_total_sec": metrics_total_sec,
                "context_time_sec": context_time_sec,
                "context_total_sec": context_total_sec,
                "validation_total_sec": validation_total_sec,
                "workflow_total_sec": workflow_total_sec,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.run_perf_context_time_sec import RunPerfContextTimeSec
        from ..models.run_perf_metrics_time_sec import RunPerfMetricsTimeSec

        d = src_dict.copy()
        init_total_sec = d.pop("init_total_sec")

        metrics_time_sec = RunPerfMetricsTimeSec.from_dict(d.pop("metrics_time_sec"))

        metrics_total_sec = d.pop("metrics_total_sec")

        context_time_sec = RunPerfContextTimeSec.from_dict(d.pop("context_time_sec"))

        context_total_sec = d.pop("context_total_sec")

        validation_total_sec = d.pop("validation_total_sec")

        workflow_total_sec = d.pop("workflow_total_sec")

        run_perf = cls(
            init_total_sec=init_total_sec,
            metrics_time_sec=metrics_time_sec,
            metrics_total_sec=metrics_total_sec,
            context_time_sec=context_time_sec,
            context_total_sec=context_total_sec,
            validation_total_sec=validation_total_sec,
            workflow_total_sec=workflow_total_sec,
        )

        run_perf.additional_properties = d
        return run_perf

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.debug_llm_validate_request_additional_data import DebugLLMValidateRequestAdditionalData
    from ..models.input_context import InputContext
    from ..models.run_options import RunOptions


T = TypeVar("T", bound="DebugLLMValidateRequest")


@_attrs_define
class DebugLLMValidateRequest:
    """
    Attributes:
        dataset_id (str):
        policy (str):
        prompt (Union[None, Unset, str]):
        response (Union[None, Unset, str]):
        context (Union['InputContext', None, Unset]):
        id (Union[None, Unset, str]):
        timestamp (Union[Unset, int]):
        additional_data (Union[Unset, DebugLLMValidateRequestAdditionalData]):
        options (Union['RunOptions', None, Unset]):
    """

    dataset_id: str
    policy: str
    prompt: Union[None, Unset, str] = UNSET
    response: Union[None, Unset, str] = UNSET
    context: Union["InputContext", None, Unset] = UNSET
    id: Union[None, Unset, str] = UNSET
    timestamp: Union[Unset, int] = UNSET
    additional_data: Union[Unset, "DebugLLMValidateRequestAdditionalData"] = UNSET
    options: Union["RunOptions", None, Unset] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.input_context import InputContext
        from ..models.run_options import RunOptions

        dataset_id = self.dataset_id

        policy = self.policy

        prompt: Union[None, Unset, str]
        if isinstance(self.prompt, Unset):
            prompt = UNSET
        else:
            prompt = self.prompt

        response: Union[None, Unset, str]
        if isinstance(self.response, Unset):
            response = UNSET
        else:
            response = self.response

        context: Union[Dict[str, Any], None, Unset]
        if isinstance(self.context, Unset):
            context = UNSET
        elif isinstance(self.context, InputContext):
            context = UNSET
            if not isinstance(self.context, Unset):
                context = self.context.to_dict()
        else:
            context = self.context

        id: Union[None, Unset, str]
        if isinstance(self.id, Unset):
            id = UNSET
        else:
            id = self.id

        timestamp = self.timestamp

        additional_data: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.additional_data, Unset):
            additional_data = self.additional_data.to_dict()

        options: Union[Dict[str, Any], None, Unset]
        if isinstance(self.options, Unset):
            options = UNSET
        elif isinstance(self.options, RunOptions):
            options = UNSET
            if not isinstance(self.options, Unset):
                options = self.options.to_dict()
        else:
            options = self.options

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "datasetId": dataset_id,
                "policy": policy,
            }
        )
        if prompt is not UNSET:
            field_dict["prompt"] = prompt
        if response is not UNSET:
            field_dict["response"] = response
        if context is not UNSET:
            field_dict["context"] = context
        if id is not UNSET:
            field_dict["id"] = id
        if timestamp is not UNSET:
            field_dict["timestamp"] = timestamp
        if additional_data is not UNSET:
            field_dict["additional_data"] = additional_data
        if options is not UNSET:
            field_dict["options"] = options

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.debug_llm_validate_request_additional_data import DebugLLMValidateRequestAdditionalData
        from ..models.input_context import InputContext
        from ..models.run_options import RunOptions

        d = src_dict.copy()
        dataset_id = d.pop("datasetId")

        policy = d.pop("policy")

        def _parse_prompt(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        prompt = _parse_prompt(d.pop("prompt", UNSET))

        def _parse_response(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        response = _parse_response(d.pop("response", UNSET))

        def _parse_context(data: object) -> Union["InputContext", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                _context_type_0 = data
                context_type_0: Union[Unset, InputContext]
                if isinstance(_context_type_0, Unset):
                    context_type_0 = UNSET
                else:
                    context_type_0 = InputContext.from_dict(_context_type_0)

                return context_type_0
            except:  # noqa: E722
                pass
            return cast(Union["InputContext", None, Unset], data)

        context = _parse_context(d.pop("context", UNSET))

        def _parse_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        id = _parse_id(d.pop("id", UNSET))

        timestamp = d.pop("timestamp", UNSET)

        _additional_data = d.pop("additional_data", UNSET)
        additional_data: Union[Unset, DebugLLMValidateRequestAdditionalData]
        if isinstance(_additional_data, Unset):
            additional_data = UNSET
        else:
            additional_data = DebugLLMValidateRequestAdditionalData.from_dict(_additional_data)

        def _parse_options(data: object) -> Union["RunOptions", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                _options_type_0 = data
                options_type_0: Union[Unset, RunOptions]
                if isinstance(_options_type_0, Unset):
                    options_type_0 = UNSET
                else:
                    options_type_0 = RunOptions.from_dict(_options_type_0)

                return options_type_0
            except:  # noqa: E722
                pass
            return cast(Union["RunOptions", None, Unset], data)

        options = _parse_options(d.pop("options", UNSET))

        debug_llm_validate_request = cls(
            dataset_id=dataset_id,
            policy=policy,
            prompt=prompt,
            response=response,
            context=context,
            id=id,
            timestamp=timestamp,
            additional_data=additional_data,
            options=options,
        )

        debug_llm_validate_request.additional_properties = d
        return debug_llm_validate_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="LoggerStatusResponse")


@_attrs_define
class LoggerStatusResponse:
    """
    Attributes:
        dataset_timestamps (int):
        dataset_profiles (int):
        segment_caches (int):
        writers (int):
        pending_writables (int):
        pending_views (List[str]):
        views (List[str]):
    """

    dataset_timestamps: int
    dataset_profiles: int
    segment_caches: int
    writers: int
    pending_writables: int
    pending_views: List[str]
    views: List[str]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        dataset_timestamps = self.dataset_timestamps

        dataset_profiles = self.dataset_profiles

        segment_caches = self.segment_caches

        writers = self.writers

        pending_writables = self.pending_writables

        pending_views = self.pending_views

        views = self.views

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "dataset_timestamps": dataset_timestamps,
                "dataset_profiles": dataset_profiles,
                "segment_caches": segment_caches,
                "writers": writers,
                "pending_writables": pending_writables,
                "pending_views": pending_views,
                "views": views,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        dataset_timestamps = d.pop("dataset_timestamps")

        dataset_profiles = d.pop("dataset_profiles")

        segment_caches = d.pop("segment_caches")

        writers = d.pop("writers")

        pending_writables = d.pop("pending_writables")

        pending_views = cast(List[str], d.pop("pending_views"))

        views = cast(List[str], d.pop("views"))

        logger_status_response = cls(
            dataset_timestamps=dataset_timestamps,
            dataset_profiles=dataset_profiles,
            segment_caches=segment_caches,
            writers=writers,
            pending_writables=pending_writables,
            pending_views=pending_views,
            views=views,
        )

        logger_status_response.additional_properties = d
        return logger_status_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
